@@ -5,37 +5,17 @@ use Carp 'croak';
 use Exporter 'import';
 use List::Util;
 use Mojo::ByteStream;
-use Mojo::Util 'deprecated';
 use Scalar::Util 'blessed';
 
-# DEPRECATED in Tiger Face!
-use overload '""' => sub {
-  deprecated 'Stringification support in Mojo::Collection is DEPRECATED'
-    . ' in favor of Mojo::Collection::join';
-  shift->join("\n");
-};
-use overload bool => sub {1}, fallback => 1;
-
 our @EXPORT_OK = ('c');
 
-# DEPRECATED in Tiger Face!
-sub AUTOLOAD {
-  my $self = shift;
-  my ($package, $method) = our $AUTOLOAD =~ /^(.+)::(.+)$/;
-  deprecated "Mojo::Collection::AUTOLOAD ($method) is DEPRECATED"
-    . ' in favor of Mojo::Collection::map';
-  croak "Undefined subroutine &${package}::$method called"
-    unless blessed $self && $self->isa(__PACKAGE__);
-  return $self->map($method, @_);
-}
-
-# DEPRECATED in Tiger Face!
-sub DESTROY { }
+sub TO_JSON { [@{shift()}] }
 
 sub c { __PACKAGE__->new(@_) }
 
 sub compact {
-  $_[0]->new(grep { defined && (ref || length) } @{$_[0]});
+  my $self = shift;
+  return $self->new(grep { defined && (ref || length) } @$self);
 }
 
 sub each {
@@ -47,18 +27,18 @@ sub each {
 }
 
 sub first {
-  my ($self, $cb) = @_;
+  my ($self, $cb) = (shift, shift);
   return $self->[0] unless $cb;
-  return List::Util::first { $cb->($_) } @$self if ref $cb eq 'CODE';
-  return List::Util::first { $_ =~ $cb } @$self;
+  return List::Util::first { $_ =~ $cb } @$self if ref $cb eq 'Regexp';
+  return List::Util::first { $_->$cb(@_) } @$self;
 }
 
 sub flatten { $_[0]->new(_flatten(@{$_[0]})) }
 
 sub grep {
-  my ($self, $cb) = @_;
-  return $self->new(grep { $cb->($_) } @$self) if ref $cb eq 'CODE';
-  return $self->new(grep { $_ =~ $cb } @$self);
+  my ($self, $cb) = (shift, shift);
+  return $self->new(grep { $_ =~ $cb } @$self) if ref $cb eq 'Regexp';
+  return $self->new(grep { $_->$cb(@_) } @$self);
 }
 
 sub join {
@@ -77,14 +57,6 @@ sub new {
   return bless [@_], ref $class || $class;
 }
 
-# DEPRECATED in Tiger Face!
-sub pluck {
-  deprecated
-    'Mojo::Collection::pluck is DEPRECATED in favor of Mojo::Collection::map';
-  my ($self, $key) = (shift, shift);
-  return $self->new(map { ref eq 'HASH' ? $_->{$key} : $_->$key(@_) } @$self);
-}
-
 sub reduce {
   my $self = shift;
   @_ = (@_, @$self);
@@ -118,9 +90,13 @@ sub sort {
 
 sub tap { shift->Mojo::Base::tap(@_) }
 
+sub to_array { [@{shift()}] }
+
 sub uniq {
+  my ($self, $cb) = (shift, shift);
   my %seen;
-  return $_[0]->new(grep { !$seen{$_}++ } @{$_[0]});
+  return $self->new(grep { !$seen{$_->$cb(@_)}++ } @$self) if $cb;
+  return $self->new(grep { !$seen{$_}++ } @$self);
 }
 
 sub _flatten {
@@ -148,8 +124,8 @@ Mojo::Collection - Collection
 
   # Chain methods
   $collection->map(sub { ucfirst })->shuffle->each(sub {
-    my ($word, $count) = @_;
-    say "$count: $word";
+    my ($word, $num) = @_;
+    say "$num: $word";
   });
 
   # Use the alternative constructor
@@ -180,6 +156,12 @@ Construct a new array-based L<Mojo::Collection> object.
 
 L<Mojo::Collection> implements the following methods.
 
+=head2 TO_JSON
+
+  my $array = $collection->TO_JSON;
+
+Alias for L</"to_array">.
+
 =head2 compact
 
   my $new = $collection->compact;
@@ -187,6 +169,9 @@ L<Mojo::Collection> implements the following methods.
 Create a new collection with all elements that are defined and not an empty
 string.
 
+  # "0, 1, 2, 3"
+  Mojo::Collection->new(0, 1, undef, 2, '', 3)->compact->join(', ');
+
 =head2 each
 
   my @elements = $collection->each;
@@ -198,8 +183,8 @@ to the callback and is also available as C<$_>.
 
   # Make a numbered list
   $collection->each(sub {
-    my ($e, $count) = @_;
-    say "$count: $e";
+    my ($e, $num) = @_;
+    say "$num: $e";
   });
 
 =head2 first
@@ -207,11 +192,19 @@ to the callback and is also available as C<$_>.
   my $first = $collection->first;
   my $first = $collection->first(qr/foo/);
   my $first = $collection->first(sub {...});
+  my $first = $collection->first($method);
+  my $first = $collection->first($method, @args);
 
-Evaluate regular expression or callback for each element in collection and
-return the first one that matched the regular expression, or for which the
-callback returned true. The element will be the first argument passed to the
-callback and is also available as C<$_>.
+Evaluate regular expression/callback for, or call method on, each element in
+collection and return the first one that matched the regular expression, or for
+which the callback/method returned true. The element will be the first argument
+passed to the callback and is also available as C<$_>.
+
+  # Longer version
+  my $first = $collection->first(sub { $_->$method(@args) });
+
+  # Find first value that contains the word "mojo"
+  my $interesting = $collection->first(qr/mojo/i);
 
   # Find first value that is greater than 5
   my $greater = $collection->first(sub { $_ > 5 });
@@ -223,19 +216,31 @@ callback and is also available as C<$_>.
 Flatten nested collections/arrays recursively and create a new collection with
 all elements.
 
+  # "1, 2, 3, 4, 5, 6, 7"
+  Mojo::Collection->new(1, [2, [3, 4], 5, [6]], 7)->flatten->join(', ');
+
 =head2 grep
 
   my $new = $collection->grep(qr/foo/);
   my $new = $collection->grep(sub {...});
+  my $new = $collection->grep($method);
+  my $new = $collection->grep($method, @args);
 
-Evaluate regular expression or callback for each element in collection and
-create a new collection with all elements that matched the regular expression,
-or for which the callback returned true. The element will be the first
-argument passed to the callback and is also available as C<$_>.
+Evaluate regular expression/callback for, or call method on, each element in
+collection and create a new collection with all elements that matched the
+regular expression, or for which the callback/method returned true. The element
+will be the first argument passed to the callback and is also available as
+C<$_>.
+
+  # Longer version
+  my $new = $collection->grep(sub { $_->$method(@args) });
 
   # Find all values that contain the word "mojo"
   my $interesting = $collection->grep(qr/mojo/i);
 
+  # Find all values that are greater than 5
+  my $greater = $collection->grep(sub { $_ > 5 });
+
 =head2 join
 
   my $stream = $collection->join;
@@ -258,9 +263,9 @@ Return the last element in collection.
   my $new = $collection->map($method);
   my $new = $collection->map($method, @args);
 
-Evaluate callback for, or call method on, each element in collection and
-create a new collection from the results. The element will be the first
-argument passed to the callback and is also available as C<$_>.
+Evaluate callback for, or call method on, each element in collection and create
+a new collection from the results. The element will be the first argument
+passed to the callback and is also available as C<$_>.
 
   # Longer version
   my $new = $collection->map(sub { $_->$method(@args) });
@@ -300,6 +305,9 @@ Create a new collection with all elements in reverse order.
 
 Create a new collection with all selected elements.
 
+  # "B C E"
+  Mojo::Collection->new('A', 'B', 'C', 'D', 'E')->slice(1, 2, 4)->join(' ');
+
 =head2 shuffle
 
   my $new = $collection->shuffle;
@@ -320,8 +328,8 @@ Number of elements in collection.
 Sort elements based on return value of callback and create a new collection
 from the results.
 
-  # Sort values case insensitive
-  my $insensitive = $collection->sort(sub { uc($a) cmp uc($b) });
+  # Sort values case-insensitive
+  my $case_insensitive = $collection->sort(sub { uc($a) cmp uc($b) });
 
 =head2 tap
 
@@ -329,11 +337,31 @@ from the results.
 
 Alias for L<Mojo::Base/"tap">.
 
+=head2 to_array
+
+  my $array = $collection->to_array;
+
+Turn collection into array reference.
+
 =head2 uniq
 
   my $new = $collection->uniq;
+  my $new = $collection->uniq(sub {...});
+  my $new = $collection->uniq($method);
+  my $new = $collection->uniq($method, @args);
 
-Create a new collection without duplicate elements.
+Create a new collection without duplicate elements, using the string
+representation of either the elements or the return value of the
+callback/method.
+
+  # Longer version
+  my $new = $collection->uniq(sub { $_->$method(@args) });
+
+  # "foo bar baz"
+  Mojo::Collection->new('foo', 'bar', 'bar', 'baz')->uniq->join(' ');
+
+  # "[[1, 2], [2, 1]]"
+  Mojo::Collection->new([1, 2], [2, 1], [3, 2])->uniq(sub{ $_->[1] })->to_array;
 
 =head1 SEE ALSO
 
