@@ -2,20 +2,21 @@ package Mojo::IOLoop::Client;
 use Mojo::Base 'Mojo::EventEmitter';
 
 use Errno 'EINPROGRESS';
-use IO::Socket::INET;
+use IO::Socket::IP;
 use Mojo::IOLoop;
 use Scalar::Util 'weaken';
-use Socket qw(IPPROTO_TCP TCP_NODELAY);
+use Socket qw(IPPROTO_TCP SOCK_STREAM TCP_NODELAY);
 
-# IPv6 support requires IO::Socket::IP
-use constant IPV6 => $ENV{MOJO_NO_IPV6}
+# Non-blocking name resolution requires Net::DNS::Native
+use constant NDN => $ENV{MOJO_NO_NDN}
   ? 0
-  : eval 'use IO::Socket::IP 0.20 (); 1';
+  : eval 'use Net::DNS::Native 0.15 (); 1';
+my $NDN = NDN ? Net::DNS::Native->new(pool => 5, extra_thread => 1) : undef;
 
 # TLS support requires IO::Socket::SSL
 use constant TLS => $ENV{MOJO_NO_TLS}
   ? 0
-  : eval 'use IO::Socket::SSL 1.84 (); 1';
+  : eval 'use IO::Socket::SSL 1.94 (); 1';
 use constant TLS_READ  => TLS ? IO::Socket::SSL::SSL_WANT_READ()  : 0;
 use constant TLS_WRITE => TLS ? IO::Socket::SSL::SSL_WANT_WRITE() : 0;
 
@@ -31,16 +32,42 @@ has reactor => sub { Mojo::IOLoop->singleton->reactor };
 sub DESTROY { shift->_cleanup }
 
 sub connect {
-  my $self = shift;
-  my $args = ref $_[0] ? $_[0] : {@_};
+  my ($self, $args) = (shift, ref $_[0] ? $_[0] : {@_});
+
+  # Timeout
   weaken $self;
-  $self->reactor->next_tick(sub { $self && $self->_connect($args) });
+  my $reactor = $self->reactor;
+  $self->{timer} = $reactor->timer($args->{timeout} || 10,
+    sub { $self->emit(error => 'Connect timeout') });
+
+  # Blocking name resolution
+  $_ && s/[[\]]//g for @$args{qw(address socks_address)};
+  my $address = $args->{socks_address} || ($args->{address} ||= '127.0.0.1');
+  return $reactor->next_tick(sub { $self && $self->_connect($args) })
+    if !NDN || $args->{handle};
+
+  # Non-blocking name resolution
+  my $handle = $self->{dns} = $NDN->getaddrinfo($address, _port($args),
+    {protocol => IPPROTO_TCP, socktype => SOCK_STREAM});
+  $reactor->io(
+    $handle => sub {
+      my $reactor = shift;
+
+      $reactor->remove($self->{dns});
+      my ($err, @res) = $NDN->get_result(delete $self->{dns});
+      return $self->emit(error => "Can't resolve: $err") if $err;
+
+      $args->{addr_info} = \@res;
+      $self->_connect($args);
+    }
+  )->watch($handle, 1, 0);
 }
 
 sub _cleanup {
   my $self = shift;
-  return $self unless my $reactor = $self->reactor;
-  $self->{$_} && $reactor->remove(delete $self->{$_}) for qw(timer handle);
+  $NDN->timedout($self->{dns}) if $self->{dns};
+  my $reactor = $self->reactor;
+  $self->{$_} && $reactor->remove(delete $self->{$_}) for qw(dns timer handle);
   return $self;
 }
 
@@ -48,39 +75,33 @@ sub _connect {
   my ($self, $args) = @_;
 
   my $handle;
-  my $reactor = $self->reactor;
-  my $address = $args->{socks_address} || ($args->{address} ||= 'localhost');
-  my $port = $args->{socks_port} || $args->{port} || ($args->{tls} ? 443 : 80);
+  my $address = $args->{socks_address} || $args->{address};
   unless ($handle = $self->{handle} = $args->{handle}) {
-    my %options = (
-      Blocking => 0,
-      PeerAddr => $address eq 'localhost' ? '127.0.0.1' : $address,
-      PeerPort => $port
-    );
+    my %options = (PeerAddr => $address, PeerPort => _port($args));
+    %options = (PeerAddrInfo => $args->{addr_info}) if $args->{addr_info};
+    $options{Blocking} = 0;
     $options{LocalAddr} = $args->{local_address} if $args->{local_address};
-    $options{PeerAddr} =~ s/[\[\]]//g if $options{PeerAddr};
-    my $class = IPV6 ? 'IO::Socket::IP' : 'IO::Socket::INET';
     return $self->emit(error => "Can't connect: $@")
-      unless $self->{handle} = $handle = $class->new(%options);
+      unless $self->{handle} = $handle = IO::Socket::IP->new(%options);
   }
   $handle->blocking(0);
 
-  # Timeout
-  $self->{timer} = $reactor->timer($args->{timeout} || 10,
-    sub { $self->emit(error => 'Connect timeout') });
-
-  # Wait for handle to become writable
-  weaken $self;
-  $reactor->io($handle => sub { $self->_ready($args) })->watch($handle, 0, 1);
+  $self->_wait($handle, $args);
 }
 
+sub _port { $_[0]{socks_port} || $_[0]{port} || ($_[0]{tls} ? 443 : 80) }
+
 sub _ready {
   my ($self, $args) = @_;
 
-  # Retry or handle exceptions
+  # Socket changes in between attempts and needs to be re-added for epoll/kqueue
   my $handle = $self->{handle};
-  return $! == EINPROGRESS ? undef : $self->emit(error => $!)
-    if $handle->isa('IO::Socket::IP') && !$handle->connect;
+  if ($handle->isa('IO::Socket::IP') && !$handle->connect) {
+    return $self->emit(error => $!) unless $! == EINPROGRESS;
+    $self->reactor->remove($handle);
+    return $self->_wait($handle, $args);
+  }
+
   return $self->emit(error => $! || 'Not connected') unless $handle->connected;
 
   # Disable Nagle's algorithm
@@ -122,11 +143,10 @@ sub _try_socks {
   my $handle = $self->{handle};
   return $self->_try_tls($args) unless $args->{socks_address};
   return $self->emit(
-    error => 'IO::Socket::Socks 0.64 required for SOCKS support')
+    error => 'IO::Socket::Socks 0.64+ required for SOCKS support')
     unless SOCKS;
 
-  my %options
-    = (ConnectAddr => $args->{address}, ConnectPort => $args->{port});
+  my %options = (ConnectAddr => $args->{address}, ConnectPort => $args->{port});
   @options{qw(AuthType Username Password)}
     = ('userpass', @$args{qw(socks_user socks_pass)})
     if $args->{socks_user};
@@ -142,9 +162,8 @@ sub _try_tls {
   my ($self, $args) = @_;
 
   my $handle = $self->{handle};
-  return $self->_cleanup->emit(connect => $handle)
-    if !$args->{tls} || $handle->isa('IO::Socket::SSL');
-  return $self->emit(error => 'IO::Socket::SSL 1.84 required for TLS support')
+  return $self->_cleanup->emit(connect => $handle) unless $args->{tls};
+  return $self->emit(error => 'IO::Socket::SSL 1.94+ required for TLS support')
     unless TLS;
 
   # Upgrade
@@ -168,6 +187,13 @@ sub _try_tls {
   $reactor->io($handle => sub { $self->_tls })->watch($handle, 0, 1);
 }
 
+sub _wait {
+  my ($self, $handle, $args) = @_;
+  weaken $self;
+  $self->reactor->io($handle => sub { $self->_ready($args) })
+    ->watch($handle, 0, 1);
+}
+
 1;
 
 =encoding utf8
@@ -243,8 +269,9 @@ implements the following new ones.
 
   $client->connect(address => '127.0.0.1', port => 3000);
 
-Open a socket connection to a remote host. Note that TLS support depends on
-L<IO::Socket::SSL> (1.84+) and IPv6 support on L<IO::Socket::IP> (0.20+).
+Open a socket connection to a remote host. Note that non-blocking name
+resolution depends on L<Net::DNS::Native> (0.15+), SOCKS5 support on
+L<IO::Socket::Socks> (0.64), and TLS support on L<IO::Socket::SSL> (1.94+).
 
 These options are currently available:
 
@@ -254,7 +281,7 @@ These options are currently available:
 
   address => 'mojolicio.us'
 
-Address or host name of the peer to connect to, defaults to C<localhost>.
+Address or host name of the peer to connect to, defaults to C<127.0.0.1>.
 
 =item handle
 
