@@ -43,7 +43,7 @@ has types     => sub { Mojolicious::Types->new };
 has validator => sub { Mojolicious::Validator->new };
 
 our $CODENAME = 'Clinking Beer Mugs';
-our $VERSION  = '6.14';
+our $VERSION  = '6.24';
 
 sub AUTOLOAD {
   my $self = shift;
@@ -65,7 +65,6 @@ sub build_controller {
   # Embedded application
   my $stash = {};
   if (my $sub = $tx->can('stash')) { ($stash, $tx) = ($tx->$sub, $tx->tx) }
-  $stash->{'mojo.secrets'} //= $self->secrets;
 
   # Build default controller
   my $defaults = $self->defaults;
@@ -437,6 +436,9 @@ L<Mojolicious::Guides::Rendering>.
   # Add another "templates" directory
   push @{$app->renderer->paths}, '/home/sri/templates';
 
+  # Add another "templates" directory with higher precedence
+  unshift @{$app->renderer->paths}, '/home/sri/themes/blue/templates';
+
   # Add another class with templates in DATA section
   push @{$app->renderer->classes}, 'Mojolicious::Plugin::Fun';
 
@@ -497,6 +499,9 @@ L<Mojolicious::Static> object.
   # Add another "public" directory
   push @{$app->static->paths}, '/home/sri/public';
 
+  # Add another "public" directory with higher precedence
+  unshift @{$app->static->paths}, '/home/sri/themes/blue/public';
+
   # Add another class with static files in DATA section
   push @{$app->static->classes}, 'Mojolicious::Plugin::Fun';
 
@@ -944,6 +949,8 @@ Nils Diewald
 
 Oleg Zhelo
 
+Olivier Mengue
+
 Pascal Gaudette
 
 Paul Evans
